from typing import Optional
from scryfallscraper.bulk_scraping.util import BulkScraper

"""
Wanna scrape the Scryfall bulk section? Here it is!
"""


def scrape_oracle_cards(zipped: Optional[bool] = False) -> Optional[dict]:
    r"""Scrape one Scryfall card object for each Oracle ID | ~(100MB).
    https://scryfall.com/docs/api/bulk-data

    :param zipped: Return either as an dict or make a gzip in the current directory (False, Default).
    """
    with BulkScraper() as scraper:
        return scraper.scrape_data("oracle_cards", zipped=zipped)


def scrape_unique_artwork(zipped: Optional[bool] = False) -> Optional[dict]:
    r"""Scrape one Scryfall card object for unique card arts! | ~(140MB).
    https://scryfall.com/docs/api/bulk-data

    :param zipped: Return either as an dict or make a gzip in the current directory (False, Default).
    """

    with BulkScraper() as scraper:
        return scraper.scrape_data("unique_artwork", zipped=zipped)


def scrape_default_cards(zipped: Optional[bool] = False) -> Optional[dict]:
    r"""Scrape every single card object, returns English or the only printed language. | ~(300MB).
    https://scryfall.com/docs/api/bulk-data

    :param zipped: Return either as an dict or make a gzip in the current directory (False, Default).
    """
    with BulkScraper() as scraper:
        return scraper.scrape_data("default_cards", zipped=zipped)


def scrape_all_cards(zipped: Optional[bool] = False) -> Optional[dict]:
    r"""Scrape one Scryfall card object for each card, in each language (when possible) | ~(1.50GB).
    https://scryfall.com/docs/api/bulk-data

    :param zipped: Return either as an dict or make a gzip in the current directory (False, Default).
    """
    with BulkScraper() as scraper:
        return scraper.scrape_data("all_cards", zipped=zipped)


def scrape_rulings(zipped: Optional[bool] = False) -> Optional[dict]:
    r"""A JSON file containing all Rulings on Scryfall. Each ruling refers to cards via an `oracle_id`. | ~(15MB).
    https://scryfall.com/docs/api/bulk-data

    :param zipped: Return either as an dict or make a gzip in the current directory (False, Default).
    """
    with BulkScraper() as scraper:
        return scraper.scrape_data("rulings", zipped=zipped)



# from scryfall_scraper_scripts import bulk_scraping

# # bulk_scraping.scrape_oracle_cards(params={})
# from scryfall_scraper_scripts import scraper

# # with scraper.BulkScraper() as bulk:
# #     bulk.s
# def scrape_oracle_cards(params=None, **kwargs):
#     r""" Scrape oracle data.

#     :param bulk_type: adasdsd
#     :param zipped: Return either as an dict or write to the current directory.
#     """
#     bulk = scraper.BulkScraper()
#     bulk.scrape_bulk_data("oracle", params=params, **kwargs)

from scryfallscraper import sets

print(sets.scrape_by_code("mmq"))


import json
import os
from typing import Union, Optional
from scryfallscraper import scraper


def scrape_all_sets(zipped: Optional[bool] = False):
    r"""Scrape set data: https://scryfall.com/docs/api/sets/all

    :param zipped: Return either as an dict or make a gzip in the current directory (False, Default).
    """

    with scraper.Scraper(
        url=f"https://api.scryfall.com/sets/",
        scrape_type="sets",
        zipped=zipped,
    ) as set_scrape:
        return set_scrape.scrape_data()


def scrape_by_code(set: str, zipped: Optional[bool] = False):
    r"""Scrape set data by code: https://scryfall.com/docs/api/sets/code


    :param zipped: Return either as an dict or make a gzip in the current directory (False, Default).
    """
    set_len = len(set)
    if set_len > 5 or set_len < 3:
        raise ValueError("Set Name is Too Short!")

    with scraper.Scraper(
        url=f"https://api.scryfall.com/sets/{set}",
        scrape_type="sets",
        zipped=zipped,
    ) as set_scrape:
        return set_scrape.scrape_data()


def scrape_by_tcgplayer_id(tcgplayer_id: str, zipped: Optional[bool] = False):
    r"""Scrape set data by TCGPlayer ID: https://scryfall.com/docs/api/sets/tcgplayer


    :param zipped: Return either as an dict or make a gzip in the current directory (False, Default).
    """

    with scraper.Scraper(
        url=f"https://api.scryfall.com/sets/tcgplayer/{tcgplayer_id}",
        scrape_type="sets",
        zipped=zipped,
    ) as set_scrape:
        return set_scrape.scrape_data()


def scrape_by_scryfall_id(scryfall_id: str, zipped: Optional[bool] = False):
    r"""Scrape set data by TCGPlayer ID: https://scryfall.com/docs/api/sets/tcgplayer


    :param zipped: Return either as an dict or make a gzip in the current directory (False, Default).
    """
    scry_length = len(scryfall_id)
    if scry_length < 10:
        # ? So I don't know if there is a standardized format for it.
        raise ValueError("Set Name is Too Short!")

    with scraper.Scraper(
        url=f"https://api.scryfall.com/sets/{scryfall_id}",
        scrape_type="sets",
        zipped=zipped,
    ) as set_scrape:
        return set_scrape.scrape_data()



import re
import requests
import shutil
import gzip
import json
from typing import Union, Optional


def send_request(url: str) -> Union[list, dict]:
    """
    Get some data from Scryfall.
    """
    resp = requests.get(url)
    if not resp.status_code == 200:
        raise ConnectionError(resp.text)
    return resp.json()


def parse_time(time: str) -> Optional[str]:
    """
    Make the last updated time return either a filename friendly string, or None.
    """
    # Match the time.
    regex_pattern = re.compile(
        r"[0-9]{4}-[0-9]{2}-[0-9]{2}.*[0-9]{2}:[0-9]{2}:[0-9]{2}", re.IGNORECASE
    )
    fetch_time = regex_pattern.search(time)
    if fetch_time:
        fn_replacers: list[tuple[str, str]] = [("T", "_"), (":", ""), (" ", "_")]
        file_name = fetch_time.group()
        for rm, new in fn_replacers:
            file_name = file_name.replace(rm, new)
        return file_name
    return None


def compress_obj(data: dict, path: str):
    path = f"{path}.gz"
    data_bytes = json.dumps(data).encode("utf-8")
    with gzip.open(f"data/{path}", "wb") as compressed_fp:
        compressed_fp.write(data_bytes)



r"""
Scrape Bulk Magic Card Data from Scryfall!
Warning: Most data here is > 100MB.

Documentation: https://scryfall.com/docs/api/bulk-data
"""
from .functions import (
    scrape_oracle_cards,
    scrape_unique_artwork,
    scrape_default_cards,
    scrape_all_cards,
    scrape_rulings,
)



"""
Do you really like Scryfall API? Yeah, same.

bulk_scraping: Scrape from Scryfall's large bulk data.
sets: Scrape set metadata.
"""

# from importlib.metadata import version

# try:
#     __version__ = version("scryfallscraper")
# except:
#     __version__ = "UNKNOWN"


import traceback

try:
    from scryfallscraper.bulk_scraping import functions as bulk_scraping
    from scryfallscraper.compression import functions as compression
    from scryfallscraper.sets import functions as scrape_sets
except Exception:
    traceback.print_exc()



import json
import os
import datetime
from typing import Union, Optional
from scryfallscraper.util import util


class Scraper:
    r"""
    :param zipped: Return either zipped up or as an object
    :param scrape_type: What sort of scrape is it? Used for files
    :param url: The URL

    """

    def __init__(
        self,
        scrape_type: str,
        url: str,
        zipped: bool = False,
    ) -> None:
        self.zipped = zipped
        self.scrape_type = scrape_type
        self.url = url

    def get_bulk_data_link(self) -> dict:
        if w := util.send_request(self.url):
            return w

    def write_json_data(self, data: Union[dict, list], path: str) -> None:
        with open(path, "w") as bulk_data_fp:
            json.dump(data, bulk_data_fp)

    def check_dir_exist(self) -> None:
        path: str = f"data/{self.scrape_type}"
        if not os.path.exists(path):
            os.makedirs(path)

    def scrape_data(self) -> Optional[dict]:
        r"""
        All-Purpose Scraper!
        """
        data = self.get_bulk_data_link()
        if self.zipped:
            file_name = f"{self.scrape_type}/{util.parse_time(str(datetime.datetime.now(datetime.timezone.utc)))}"
            self.check_dir_exist()
            util.compress_obj(data, file_name)
        else:
            return data

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, tb):
        pass



import json
import os
import datetime
import traceback
from typing import Union, Optional
from scryfallscraper.util import util


class BulkScraper:
    r"""

    :param zipped: Return either zipped up or as an object
    :param scrape_type: What sort of scrape is it? Used for files
    :param url: The URL

    """

    def __init__(self) -> None:
        self.bulk_data_links = util.send_request("https://api.scryfall.com/bulk-data")[
            "data"
        ]

    def write_json_data(self, data: Union[dict, list], path: str) -> None:
        with open(path, "w") as bulk_data_fp:
            json.dump(data, bulk_data_fp)

    def check_dir_exist(self, bulk_type: str) -> None:
        if not os.path.exists(f"data/{bulk_type}"):
            os.makedirs(f"data/{bulk_type}")

    def scrape_data(self, bulk_type: str, zipped: bool = True) -> Optional[dict]:
        r"""Scrape the data

        :param zipped: Return either as an dict or write to the current directory.
        """

        for bulk_data_info in self.bulk_data_links:
            if bulk_data_info["type"] == bulk_type:
                file_name: str = f"{util.parse_time(bulk_data_info['updated_at'])}"
                path = f"{bulk_type}/{file_name}"
                scraped_data = util.send_request(bulk_data_info["download_uri"])

                self.check_dir_exist(bulk_type)
                if zipped:
                    util.compress_obj(scraped_data, path)
                else:
                    return scraped_data
                break

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, tb):
        pass



r"""
Scrape Scryfall set data!

Documentation: https://scryfall.com/docs/api/sets
:
"""
from .functions import (
    scrape_by_code,
    scrape_by_scryfall_id,
    scrape_all_sets,
    scrape_by_tcgplayer_id,
)



import gzip
import shutil
import os
import glob
from typing import Optional


def compress_dir(zip_path: Optional[str] = "", f_path: Optional[str] = "") -> None:
    """
    Compress a whole directory! I hope it works!

    f_path: Location of the .json files to compress. Default current directory.
    zip_path: Location of the .gz file. Default current directory.
    """
    if not f_path:
        f_path = ""

    files: list = [os.path.basename(files) for files in glob.glob(f"{f_path}*.json")]
    if not os.path.exists(zip_path):
        zip_path = "./"

    for file in files:
        file_path = f"{zip_path}{file}.gz"
        if not os.path.exists(file_path):
            with gzip.open(file_path, "wb") as compressed_fp:
                shutil.copyfileobj(open(f"{f_path}{file}", "rb"), compressed_fp)
